{ FoxClone
  Copyright (C) 2023 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}

{--------------------------------------------------------------------------------------------------------
FOXCLONE51

Compiling in a later version of ubuntu, e.g. LM21 and then using in an iso built from an earlier
ubuntu, e.g. 18.04, generates a run time error:

    /lib/x86_64-linux-gnu/libc.so.6 version 'GLIBC_2.34' not found.

Solution found here:
    https://forum.lazarus.freepascal.org/index.php/topic,58888.msg483626.html#msg483626

To find existing dependencies:

andy@T432 ~/foxclone51 $ readelf --dyn-syms -W foxclone51 | grep "GLIBC"
    27: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND wcrtomb@GLIBC_2.2.5 (2)
    85: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND wcscoll@GLIBC_2.2.5 (2)
   160: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND dlerror@GLIBC_2.34 (3)
   252: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND setenv@GLIBC_2.2.5 (2)
   373: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND setlocale@GLIBC_2.2.5 (2)
   374: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND strcoll@GLIBC_2.2.5 (2)
   412: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND dlopen@GLIBC_2.34 (3)
   436: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND iconv_close@GLIBC_2.2.5 (2)
   489: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND mbrlen@GLIBC_2.2.5 (2)
   521: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND dlclose@GLIBC_2.34 (3)
   532: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND mbrtowc@GLIBC_2.2.5 (2)
   598: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND __libc_start_main@GLIBC_2.34 (3)
   702: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND dlsym@GLIBC_2.34 (3)
   743: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND iconv@GLIBC_2.2.5 (2)
   841: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND iconv_open@GLIBC_2.2.5 (2)
   880: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND sysconf@GLIBC_2.2.5 (2)
   983: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND nl_langinfo@GLIBC_2.2.5 (2)
  1054: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND towlower@GLIBC_2.2.5 (2)
  1123: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND dladdr@GLIBC_2.34 (3)
  1189: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND __errno_location@GLIBC_2.2.5 (2)
  1260: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND towupper@GLIBC_2.2.5 (2)
  1290: 0000000000000000     0 FUNC    WEAK   DEFAULT  UND __cxa_finalize@GLIBC_2.2.5 (2)

now Create a fake libdl.so file using the following libdl.c:

    void dlopen() { }
    void dlsym() { }
    void dladdr() { }
    void dlclose() { }
    void dlerror() { }
    void __libc_start_main() { }

then
       gcc -fPIC -shared -o libdl.so -Wl,--soname='libdl.so.2' libdl.c

Now just place the fake libdl.so into your project directory before compiling. Nothing more is required.

libdl.so is in the foxclone51 folder

recompile

andy@T432 ~/foxclone51 $ readelf --dyn-syms -W foxclone51 | grep "GLIBC"
    27: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND wcrtomb@GLIBC_2.2.5 (2)
    85: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND wcscoll@GLIBC_2.2.5 (2)
   252: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND setenv@GLIBC_2.2.5 (2)
   373: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND setlocale@GLIBC_2.2.5 (2)
   374: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND strcoll@GLIBC_2.2.5 (2)
   435: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND iconv_close@GLIBC_2.2.5 (2)
   488: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND mbrlen@GLIBC_2.2.5 (2)
   530: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND mbrtowc@GLIBC_2.2.5 (2)
   740: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND iconv@GLIBC_2.2.5 (2)
   839: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND iconv_open@GLIBC_2.2.5 (2)
   878: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND sysconf@GLIBC_2.2.5 (2)
   982: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND nl_langinfo@GLIBC_2.2.5 (2)
  1053: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND towlower@GLIBC_2.2.5 (2)
  1188: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND __errno_location@GLIBC_2.2.5 (2)
  1259: 0000000000000000     0 FUNC    GLOBAL DEFAULT  UND towupper@GLIBC_2.2.5 (2)
  1290: 0000000000000000     0 FUNC    WEAK   DEFAULT  UND __cxa_finalize@GLIBC_2.2.5 (2)

no GLIBC_2.34, tested and works.

Foxclone51:

   fixes the problem with LM21 legacy erase and install where it fails to copy grub. Because it is a
   GPT partition table it does not copy the first 446 bytes. Now fixed, the old grub function replaced
   with separate functions for backup, restore, etc.

   fixes a problem where foxclone would error when reading drives if more than 9 partitions on a drive
   problem parsing sfdisk output

   sfdisk - later versions of sfdisk, i.e. in jammy, insert a new header into the dump file. If a backup
   is taken with a version of foxclone running under jammy or a derivative, e.g. mint 21, and then this
   backup is then restored with a version of foxclone booting from an earlier iso, e.g. bionic or focal,
   then it will crash. The earlier version of sfdisk does not recognise the sector-size line.

   To prevent any possibility of conflict, foxclone checks the sfdisk dump file for a sector-size line
   and removes it.

grub backup restore etc
legacy lm21 can select parts?
same on backup drive?
verify on backup?
when saving with compress what option or cancel when wont fit

Foxclone52:

   Foxclone51 fails with LUKS due to changes introduced in foxclone50 - it identifies a LUKS partition
   as "crypto_LUKS" not "unknown" which is used to work out that partclone.dd is needed.

   Fixed for backup.
   Restore done. not checked
   Clone direct done.
   clone from file not checked - checked works.

   now LVM

Foxclone53:
   fix period in filenames
   fix one drive in system
   when running in a VM will still give an error if a virtual CD has something in it, no error when
   running on bare metal.


--------------------------------------------------------------------------------------------------------}
program foxclone53;

{$mode objfpc}{$H+}

uses
  {$IFDEF UNIX}{$IFDEF UseCThreads}
  cthreads,
  {$ENDIF}{$ENDIF}
  Interfaces, // this includes the LCL widgetset
  sysutils, Forms, mainform, drives, noteform, clone, about, verify, confform,
  partclone, clocale, info;

{$R *.res}

begin
  RequireDerivedFormResource:=True;
  Application.Scaled:=True;
  Application.Initialize;
  Application.CreateForm(TMainFM, MainFm);
  Application.Run;
  //writeln('end of the main program');
  if assigned(MyDrives) then FreeAndNil(mydrives);
  if assigned(RDrive) then FreeAndNil(RDrive);
  if assigned(SysInfo) then FreeAndNil(SysInfo);
end.

